/*----------------------------------------------------------------------------------*
 :                           xhelp: A help facility for X                           :
 :                                   version 1.3                                    :    
 :				    February 1993
 :----------------------------------------------------------------------------------:
 : Copyright 1991 by Auburn University                                              :
 :                                                                                  :
 : Permission to use, copy, modify, distribute, and sell this software and its      :
 : documentation for any purpose is hereby granted without fee, provided that       :
 : the above copyright notice appear in all copies and that both that               :
 : copyright notice and this permission notice appear in supporting                 :
 : documentation, and that the name of Auburn University not be used in advertising :
 : or publicity pertaining to distribution of the software without specific,        :
 : written prior permission.  Auburn University makes no representations about the  :
 : suitability of this software for any purpose.  It is provided "as is"            :
 : without express or implied warranty.                                             :
 :                                                                                  :
 *----------------------------------------------------------------------------------*/

#include <stdio.h>
#include <X11/Xatom.h>
#define XHELP_NOT_ACTIVE 	TRUE
#define XHELP_ACTIVE   		FALSE
Atom  XHELP_FILE, XHELP_STATUS;
void XhCreateHelpAtoms(Widget);
void XhSetHelpStatusProperty(Widget, int);
int XhGetHelpStatusProperty(Widget);


/*---------------------------------------------------------------------------------------*
 :     Everything between this and the double bars below is all you'll need to use XHELP :
 *---------------------------------------------------------------------------------------*/

/*---------------------------------------------------------* 
 :  Code XHELP_INTRO_ID for the topicID when you want the  :
 :  introduction helptopic to be displayed.  XHELP         :
 :  automagically assigns this ID to the very first HELP   :
 :  topic when the help file is created.                   :
 *---------------------------------------------------------*/
#define XHELP_INTRO_ID		"welcome"   

/*----------------------------------------------*/
int XhCallXHelp(w,helpfilename, topicId)
/*----------------------------------------------*/
        Widget w;
        String helpfilename, topicId;
{
   int xhelpStatus;
   static count=0;
   char name[80], cmdstr[80];
   int result;

   if (!count) {
      XhCreateHelpAtoms(w);
      count++;
   }

   if ((topicId == NULL) ||  (strlen(topicId)==0))  
       topicId=XHELP_INTRO_ID;

   if ((xhelpStatus=XhGetHelpStatusProperty(w))==XHELP_ACTIVE) {  /** XHELP alive? **/
       sprintf(name, "%s:%s",helpfilename,topicId);
       XChangeProperty(XtDisplay(w),
             DefaultRootWindow(XtDisplay(w)),
             XHELP_FILE, XA_STRING, 8, PropModeReplace, (unsigned char *)name,
             strlen(name));
   } else { /** XHELP is not around, let's wake it up **/
         sprintf(cmdstr,"xhelp %s %s & \n",
               helpfilename,topicId); /** invoke xhelp in USER mode **/
         result=system(cmdstr);
         if (result)
            fprintf(stderr, "Could not invoke XHelp, return code =%d\n", result);
         else { 
            /*---------------------------------------------------------* 
             : It's loading.  Let's not let the user bring up multiple :
             : instances of XHELP by repeatedly pressing a help button :
             *---------------------------------------------------------*/
            XtSetSensitive(w, FALSE);
            while (XhGetHelpStatusProperty(w)==XHELP_NOT_ACTIVE);
            XtSetSensitive(w, TRUE);
         }
       }
   return xhelpStatus;  /** ignore this return code for now **/  
} /** XhCallXhelp() **/

/*--------------------------------------------------------------------------------------
 *--------------------------------------------------------------------------------------*/ 

/*--------------------------------*/
void XhSetHelpStatusProperty(w,arg)
/*--------------------------------*/
	Widget w;
        int arg;
{
     XChangeProperty(XtDisplay(w),
         DefaultRootWindow(XtDisplay(w)),
         XHELP_STATUS, 
         XA_INTEGER,
         8,
         PropModeReplace,
         (unsigned char *)&arg,
	sizeof(int));
} /** XhSetHelpStatusProperty() **/

/*---------------------*/
void XhCreateHelpAtoms(w)
/*---------------------*/
	Widget w;
{ Display * dpy = XtDisplay(w);
     XHELP_FILE = XInternAtom(dpy, "XHelp File", 0);
     XHELP_STATUS = XInternAtom(dpy, "XHelp Status", 0);
} /** CreateHelpAtoms() **/

/*------------------------*/
int XhGetHelpStatusProperty(w)
/*------------------------*/
 	Widget w;
{
   Atom actual_type;
   int  actual_format, nitems, leftover;
   int  *retdata = NULL ;

   if (XGetWindowProperty(XtDisplay(w),
            DefaultRootWindow(XtDisplay(w)),
            XHELP_STATUS,
            0L,
            (long) BUFSIZ,
            FALSE,  /** server deletes data upon receipt **/
            XA_INTEGER,
            &actual_type, &actual_format,
            (unsigned long *)&nitems,
            (unsigned long *)&leftover,
            (unsigned char **)&retdata
            ) == Success && actual_type == XA_INTEGER) {
        if (*retdata == XHELP_ACTIVE)
          return XHELP_ACTIVE;
        else
          return XHELP_NOT_ACTIVE;
    } else
        return XHELP_NOT_ACTIVE;
} /** XhGetHelpStatusProperty() **/


